/*****************************************************************************
+	TAME 1.0
+
+	formula.h
 *****************************************************************************/

#ifndef TAME_FORMULA_H
#define TAME_FORMULA_H

#include <iostream.h>
#include <fstream.h>
#include "./orclauset.h"
#include "./xorclauset.h"
#include "./assignmap.h"
#include "./selectxor.h"
#include "./vset.h"
#include "./lassign.h"

/*{\Manpage {formula} {} {Formulae}}*/

class formula {
/*{\Mdefinition
Una istanza $F$ del tipo |\Mname| \'e un insieme contenente gli elementi:
\begin {itemize}
\item {Or-clause Set}
\item {Xor-clause Set}
\item {Assignment Set}
\item {Selected-Xor Set}
\item {Variables Occurrences Vector}
\end {itemize}
}*/

  private:

/** FLAGS **/   
   bool change; //some simplification was completed successfully

/** COUNTER **/
   int lookahead; // #lookahead effectuated in the last cicle
   int calculate; // #calculate effectuated in the last cicle
   int restart; // #restart effectuated to arrive to the solution
   int solution_number; // #the number of solution found

/** Service Utility Functions ********/
  void populate_fromOR (vset & V, const orclauset & C);
  void populate_fromXOR (vset & V, const xorclauset & X);
  void populate_fromSEL (vset & V, const selectxor & S);
  void populate_fromASS (vset & V, const assignmap & Ass);
  void printsolution (ostream & os);
  void printinsolution (ostream & os);

  public:

/** SETS **/
   orclauset  C;  //or-clause-set
   xorclauset X; //xor-clause-set
   assignmap  Ass; //assignment map
   selectxor  S;   //xor-selection stack
   vset     V; // set of the all variable in the formula with occourrencies

/*{\Mcreation F }*/

   formula () {change = false;
	       lookahead = 0;
	       calculate = 0;
	       restart = 0;
	       solution_number = 0;}
/*{\Mcreate crea una istanza di |\Mvar| e la inizializza con insiemi vuoti.
}*/

/*{\Moperations 1.8 5}*/

   bool
   bottom (){return ( C.bottom() ) || (X.bottom() );}
/*{\Mop	Restituisce true se in |C| o |X| \'e presente una clausola contraddittoria, false altrimenti.}*/
   
   void
   unset_changed (){change = false;}
/*{\Mop Setta il flag che indica che ci sono stati cambiamenti a false.}*/

   void
   set_changed (){change = true;}
/*{\Mop Setta il flag che indica che ci sono stati cambiamenti a true.}*/

   bool
   changed (){return (change);}
/*{\Mop Legge il flag che indica che ci sono stati cambiamenti.}*/   

   void
   set_lookahead (int i){lookahead = i;}
/*{\Mop Valorizza il contatore del numero di lookahead con l'ultimo valore.}*/

   int
   get_lookahead (){return lookahead;}
/*{\Mop restituisce il numero di lookahead effettuati l'ultima volta.}*/

   void
   set_calculate (int i){calculate = i;}
/*{\Mop Valorizza il contatore del numero di calculate con l'ultimo valore.}*/

   int
   get_calculate (){return calculate;}
/*{\Mop Restituisce il numero di calculate effettuati l'ultima volta.}*/

  void
  set_restart (int i) {restart = i;}
/*{\Mop Valorizza il contatore del numero di restart effettuat con |i|.}*/

  int
  get_restart () {return restart;}
/*{\Mop Restituisce il numero di restart effettuati.}*/  

  int
  get_solution_number () {return solution_number;}
/*{\Mop Restituisce il numero di soluzioni finora trovate.}*/  

  bool
  resync_vset ();
/*{\Mop Risincronizza l'insieme |F.V| in base alle variabili effettivamente 
contenute nelle clausole di |F| con le occorrenze.}*/
  
  bool
  occur (var a) const;
/*{\Mop restituisce se |abs(a)| ricorre in qualche formula di |\Mvar|.}*/  

  bool
  empty() const {
/*{\Mop restituisce se gli insiemi di |\Mvar| sono tutti vuoti.}*/
    if (C.empty() &&
        X.empty() &&
	S.empty() &&
	Ass.empty() )
      return true;
    return false;
  }  

  int
  getOrPosOccurrence (const var a) const {
/*{\Mop restituisce le occorrenze positive di |a| nelle Or clausole.}*/
    vset::const_iterator itv = V.find (abs (a));
    return ( (*itv).second.get_cpos() );
  }

  int
  getOrNegOccurrence (const var a) const {
/*{\Mop restituisce le occorrenze negative di |a| nelle Or clausole.}*/
    vset::const_iterator itv = V.find (abs (a));
    return ( (*itv).second.get_cneg() );
  }

  int
  getXorOccurrence (const var a) const {
/*{\Mop restituisce le occorrenze di |a| nelle Xor clausole.}*/
    vset::const_iterator itv = V.find (abs (a));
    return ( (*itv).second.get_xor() );
  }

  int
  getSelOccurrence (const var a) const {
/*{\Mop Restituisce le occorrenze di |a| nelle |Select Xor|.}*/  
    vset::const_iterator itv = V.find (abs(a));
    return ( (*itv).second.get_sel() );
  }

  int
  getAssOccurrence (const var a) const {
/*{\Mop restituisce le occorrenze negative di |a| nelle Xor clausole.}*/
    vset::const_iterator itv = V.find (abs (a));
    return ( (*itv).second.get_ass() );
  }  

  void
  AddOr (const orclause & c);
/*{\Mop Aggiunge una clausola Or nella formula, aggiornando le occorrenze
delle variabili.}*/

  void
  AddXor (const xorclause & x);
/*{\Mop Aggiunge una clausola Xor nella formula, aggiornando le occorrenze
delle variabili.}*/

  void
  AddSel (const var v, const xorclause & x);
/*{\Mop Aggiunge una selezione Xor nella formula, aggiornando le occorrenze
delle variabili.}*/

  void
  AddAssign (var A, var B);
/*{\Mop aggiunge in |F.Ass| una nuova assegnazione, $ abs(A) := \pm B $,
eventualmente eliminando da |F.S| la definizione di |A|.}*/  

  void
  DelOr (const orclause & c);
/*{\Mop Elimina una clausola Or nella formula, aggiornando le occorrenze
delle variabili.}*/

  void
  DelOr (orclauset::iterator itc);
/*{\Mop Elimina la clausola Or nella formula, cui si riferisce l'iteratore,  
aggiornando le occorrenze delle variabili.}*/

  void
  DelXor (const xorclause & x);
/*{\Mop Elimina una clausola Xor nella formula, aggiornando le occorrenze
delle variabili.}*/

  void
  DelXor (xorclauset::iterator itx);
/*{\Mop Elimina la clausola Xor nella formula, cui si riferisce l'iteratore, 
aggiornando le occorrenze delle variabili.}*/

  void
  DelSel (selectxor::iterator its);
/*{\Mop Elimina la selezione Xor nella formula, cui si riferisce l'iteratore,
aggiornando le occorrenze delle variabili.}*/


  formula&
  operator= (const formula& G);
/*{\Mbinop    Assegna $G$ a |\Mvar| e restituisce |\Mvar|.}*/

  bool
  operator != (const formula & G);
/*{\Mbinop Ritorna se le |2| formule sono diverse.}*/  

  formula &
  operator += (const formula & G);
/*{\Mbinop    Aggiunge a |\Mvar| gli elementi di |G.C|, |G.X|, |G.S| non 
presenti in |\Mvar|.}*/  

  formula
  operator & (const formula & G);
/*{\Mbinop    Restituisce l'intersezione insiemistica di |G| e |\Mvar|.}*/  

   friend
   ostream &
   operator << (ostream & os, const formula & F);
/*{\Mbinopfunc Stampa su |os| il contenuto della formula, nell'ordine:\\
|Orclauset X|\\
|Xorclauset C|\\
|SelectXor S|\\
|Assignmap A|\\
|Variables Occurrences Vector V|\\
}*/

  void
  PrintSolution (ofstream & of);
/*{\Mop Stampa la soluzione su |of|, se aperto, altrimenti a video.}*/

  void
  PrintInsolution (ofstream & of);
/*{\Mop Stampa un messaggio di insoddisfacibilit\'a della formula su |of|, se 
aperto, altrimenti a video.}*/

}; //End "formula" Declaration

/*****************************
 Funzioni Esterne alla Classe
 *****************************/

/*{\Mtext {\bf Non-Member Functions}}*/

/*{\Mimplementation
Le Formule sono implementate tramite sets di Clause e sets di Assign. 
I tempi di esecuzione delle operazioni dipendono dalla implementazione dei 
sets.}*/

#endif
