/*****************************************************************************
+	TAME 1.0
+
+	heuristics.h
 ****************************************************************************/

#ifndef TAME_HEURISTICS_H
#define TAME_HEURISTICS_H

#include "./formula.h"
#include "./restart.h"

/*{\Manpage {Branching Strategies for Heuristics} {F} {Priority Functions}
}*/
/*** Branching Strategy ***/

  float
  MaximumSizeofAssSet (const formula & F);
/*{\Mfuncl Calcola la priorit\'a in base alla dimensione dell'insieme delle
assegnazioni della formula |F|.
Split che producono insiemi |F.Ass| pi\'u grandi portano a priorit\'a pi\'u 
elevate.
Non molto accurata: nella maggior parte dei casi corrisponde alla "First Open
Proposition".
}*/

  float
  MinimumMediumOrClauseLenght (const formula & F);
/*{\Mfuncl Calcola la priorit\'a in base alla dimensione media delle clausole
|OR|. 
Split che producono dimensioni medie maggiori portano a piorit\'a minori.
}*/

  float
  MaximalMediumOccorrenceinXor (const formula & F);
/*{\Mfuncl Calcola la priorit\'a in base al numero medio di occorrenze delle
proposizioni nelle clausole |XOR|.
Split che producono occorrenze medie maggiori portano a priorit\'a pi\'u 
elevate.
}*/

  
  float
  MinimumXorClausesMediumDimension (const formula & F);
/*{\Mfuncl Calcola la priorit\'a in base alla dimensione media delle clausole
|XOR|.
Split che producono dimensioni medie maggiori portano a piorit\'a minori.
}*/
  
  float
  MaximalAddedConstraints (const formula & F);
/*{\Mfuncl Calcola la priorit\'a in base ad un calcolo approssimativo sul numero
di |costraints| che vigono nella formula.
Il calcolo tiene conto:
\begin {itemize}
\item dei constraint dati da ogni tipo di clausola in funzione della lunghezza
\item della lunghezza media delle clausole di ogni insieme
\end {itemize}
Non vengono tracciati i |costraints| dati dalle interazioni fra letterali 
presenti in pi\'u di un insieme.
}*/

/*** End Branching Strategy ***/

class tpriority {

/*** Data Structure (See Implementation) ***/

  float
  (*P) (const formula & F); // pointer to the priority function

/*** End Data Structure ***/  

public:

  tpriority () {P = MaximumSizeofAssSet;}

  void
  set_heuristic (char * a) {
    char sigle [10];
    strcpy (sigle, a);
    if (!strcmp (sigle, "MSAS")) 
      P = MaximumSizeofAssSet;
    else 
      if (!strcmp (sigle, "MMOCL"))
        P = MinimumMediumOrClauseLenght;
    else
      if (!strcmp (sigle, "MMOXC"))
        P = MaximalMediumOccorrenceinXor;
    else
      if (!strcmp (sigle, "MXCMD"))
        P = MinimumXorClausesMediumDimension;
    else
      if (!strcmp (sigle, "MAC"))
        P = MaximalAddedConstraints;
  }

  float
  operator () (const formula & F, const var v, var & result) const {
    float pos_score, neg_score;
    formula Ftemp;
    evaluate (Ftemp, F, not (v));
    neg_score = P (Ftemp); 
    result = not (v);
    evaluate (Ftemp, F, v);
    pos_score = P (Ftemp);
    if (pos_score > neg_score) {
      result = v;
      return pos_score;
    }
    else 
      return neg_score;
  }
};  

#endif // TAME_HEURISTICS_H
