/****************************************************************************
+       TAME 1.0
+
+       lassign.h
 ****************************************************************************/

#ifndef TAME_LASSIGN_H
#define TAME_LASSIGN_H

#include <deque.h>
#include "./bassign.h"
#include "./assignmap.h"

/*{\Manpage {lassign} {} {Assegnazioni Arbitrarie di Variabili} {B}*/

class lassign : public deque <bassign> {

/*{\Mdefinition
Una istanza del tipo di dati |\Mname| rappresenta un insieme, gestito a coda-pila di assegnazioni arbitrarie di letterali a valori costanti.
Ogni assegnazione \`e contrassegnata dall'etichetta |toggable| a seconda se sia
ancora invertibile o meno. Infatti se da questa assegnazione dovesse provenire una contraddizione deve essere negata.}*/

private:

  unsigned int min_level;

public:
/*{\Mcreation B }*/
  lassign () {min_level = 0;}

  void
  set_min_level (const int level) {min_level = (unsigned int) level;}

  int
  get_min_level () {return (int) min_level;}

  void
  add (const var & LHS, const var & RHS=TRUE, bool toggle=true) {
/*{\Mop inserisce una nuova assegnazione, del tipo $LHS := RHS$.
Per default $RHS = TRUE$, l'assegnazione \'e arbitraria.
}*/
    if (LHS.is_negative()) {
      bassign ba (not (LHS), not (RHS), toggle);
      push_back (ba);
      return;
    }
    else {
      bassign ba (LHS, RHS, toggle);
      push_back (ba);
      return;
    }
  }

  bool
  Find (const var & v) const {
/*{\Mop Restituisce se esiste una assegnazione (toggable o certa) per |v|.}*/
    var aux = abs (v);
    lassign::const_iterator it;
    for (it = begin(); it != end(); it ++) 
      if ((*it).lhs == aux)
        return true;
    return false;
  }

  void
  AddCostantAssignment (const assignmap & Ass) {
/*{\Mop Aggiunge tutte le assegnazioni costanti che si trovano in |Ass|,
provenienti dai calcoli semplificativi, etichettandole come NON backtrackable.
}*/
    assignmap::const_iterator itass;
    var aux; // comodity var
    var value;
    for (itass = Ass.begin(); itass != Ass.end(); itass ++) {
      aux = (*itass).first;
      if (/*!Find (aux)*/ true) {
        value = (*itass).second;
	if (value.is_costant())
          add (aux, value, false);
      }
    } // end for all assignment in Ass
  }

  void
  Reorder () {
/*{\Mop Cancella le variabili, ultime inserite in ordine cronologico, che non
hanno settato il flag di Backtrack (ovvero sono state provate in entrambi i
valori). Inverte il segno della ultima variabile "backtrackable".}*/
    bool maden = false;
    while (
#ifdef TAME_OPTIMIZE_INITIALIZATION    
    (get_min_level() < (unsigned int) size() ) 
#endif 
           !empty() && !maden) {
      bassign last = back();
      pop_back();
      if (last.label ()) {
        last.unset_label();
        push_back (last.inverse());
	maden = true;
      }
    }
  }


  friend
  ostream &
  operator << (ostream & os, const lassign & B) {
/*{\Mbinopfunc Stampa su |os| il contenuto di |\Mvar| variabile per variabile, nel
formato: \\
|<var1>| -> |<const_value1>| [not] toggable \\
\ldots \\
|<varn>| -> |<const_valuen>| [not] toggable.
}*/
    if (B.empty())
      os << "*** Empty Arbitrary Assignment Set ***" << endl;
    for (lassign::const_iterator it = B.begin(); it != B.end(); it ++) 
      os << (*it) << endl;
    return os;
  }
  
};

#endif // TAME_LASSIGN_H
