/*****************************************************************************
+	TAME 1.0
+
+	logger.h
 *****************************************************************************/

#ifndef TAME_LOGGER_H
#define TAME_LOGGER_H

#include <iostream>
#include "../include/var.h"
#include "../include/formula.h"
#include "../include/xorque.h"

/*{\Manpage {tlogger} {} {Logging Function Object} {logger}*/

class tlogger {

/*{\Mdefinition
Una istanza del tipo |\Mname| serve per generare log informativi e ad uso debug.
Pu\'o inviare i messaggi su |3| canali distinti: |cout|, |cerr|, |auxiliary|, 
dove l'ultimo \'e un file creato da utente, attivabili separatamente, a seconda
del livello di debug impostato.
}*/

  bool std_error; // flag of output on cerr

  bool std_output; // flag of output on cout
   
  bool auxiliary; // flag of output on auxiliary stream
   
  int debug_level; // maximum debug level

  ostream *os_auxiliary; // punt to the auxiliary stream 

public:
/*{\Mcreation logger }*/

   tlogger() {
/*{\Mcreate crea una nuova istanza |\Mvar| del tipo |\Mname| abilitando |cerr|
e settando il livello di debug a |0|.}*/   
     init(true, false, 0);
   }

   tlogger(bool std_error, bool std_output) {
/*{\Mcreate crea una nuova istanza |\Mvar| del tipo |\Mname| chiedendo 
esplicitamente l'abilitazione di |cout| e |cerr| e settando il livello di debug a |0|.}*/
     init (std_error, std_output, 0);
   }

   tlogger(ostream *auxiliary) {
/*{\Mcreate crea una nuova istanza |\Mvar| del tipo |\Mname| chiedendo
esplicitamente il canale ausiliario. Disabilita |cout| e |cerr|. Setta il
livello di debug a |0|.
}*/
     init (false, false, auxiliary);
   }

/*{\Moperations 1.8 5}*/

   int 
/*{\Mop Inizializza |\Mvar| esplicitamente, abilitando o disabilitando |cout|,
|cerr|, |auxiliary| e il debug level (per default $= 0$).
}*/
   init(bool std_err, bool std_out, ostream *auxiliary, int debug_level = 0) {
     this->std_error = std_err;
     this->std_output = std_out;
     set_auxiliary(auxiliary);
     set_debug_level(debug_level);
     return 1;
   }
   
   void 
/*{\Mop abilita l'output su |cerr|.}*/   
   enable_std_error() {
     std_error = true;
   }

   void 
/*{\Mop disabilita l'output su |cerr|.}*/   
   disable_std_error() {
     std_error = false;
   }

   void 
/*{\Mop abilita l'output su |cout|.}*/   
   enable_std_output() {
     std_output = true;
   }

   void 
/*{\Mop disabilita l'output su |cout|.}*/   
   disable_std_output() {
     std_output = false;
   }

   void 
/*{\Mop abilita l'output su |auxiliary|.}*/   
   enable_auxiliary() {
     auxiliary = true;
   }

   void 
/*{\Mop disabilita l'output su |auxiliary|.}*/   
   disable_auxiliary() {
     auxiliary = false;
   }

   void 
/*{\Mop setta il canale ausiliario con l'ostream |auxiliary|.}*/   
   set_auxiliary (ostream *auxiliary) {
     os_auxiliary = auxiliary;
   }

   ostream *
/*{\Mop restituisce il puntatore allostream del canale ausiliario.}*/   
   get_auxiliary () {
     return os_auxiliary;
   }

   void 
/*{\Mop setta il livello di debug a |debug_level|.}*/   
   set_debug_level(int debug_level) {
     this->debug_level = debug_level;
   }

   int 
/*{\Mop restituisce il livello di debug.}*/   
   get_debug_level() {
     return debug_level;
   }

   void
   operator () (int debug_level, char * mesg);

   void
   
   operator () (int debug_level, char * mesg, const int i);
   
   void 
   operator () (int debug_level, char * mesg, const var & v);

   void
   operator () (int debug_level, char * mesg, const var a, const var b);

   void
   operator () (int debug_level, char * mesg, const orclause & x);

   void
   operator () (int debug_level, char * mesg, const orclauset & X);
   
   void
   operator () (int debug_level, char *, const xorclause & x);

   void
   operator () (int debug_level, char *, const xorclauset & X);

   void
   operator () (int debug_level, char * mesg, const selectxor & S);

   void
   operator () (int debug_level, char * mesg, const assignmap & Ass);
   
   void
   operator () (int debug_level, char * mesg, const lassign & VB);
   
   void 
   operator () (int debug_level, char * mesg, const formula & F);
 
   void 
   operator () (int debug_level, char * mesg, const xorque & XQ);
};

#endif // TAME_LOGGER_H
