/****************************************************************************
+	TAME 1.0
+
+	orclause.h
 *****************************************************************************/

#ifndef TAME_ORCLAUSE_H
#define TAME_ORCLAUSE_H

#include "./var.h"
#include <set.h>

/*{\Manpage {orclause} {} {Orclauses} }*/

class orclause : public set <var> {
/*{\Mdefinition
Una istanza |C| del tipo di dati |\Mname| rappresenta una clausola Or della 
logica booleana, ovvero un insieme di variabili booleane |var|.}*/

public:

/*{\Mcreation C }*/

   orclause () {}
/*{\Mcreate crea una istanza di |\Mvar| e la inizializza con insiemi vuoti.
}*/
   

/*{\Moperations 1.8 5}*/

  void 
  add (const var& x) {insert (x);}
/*{\Mop inserisce la variabile |x| nella clausola |C|.}*/


  bool
  is_complement (const orclause & D) const {
/*{\Mfunc Ritorna se le 2 clausole, che devono essere binarie, sono complementari, ovvero se data $C = \{a, b\}$ sia effettivamente $D = \{not (a), not (b)\}$
}*/
    if ((size () !=2) || (D.size () !=2)) return  false;
    orclause aux;
    for (orclause::iterator it = begin ();
         it != end ();
	 it ++)
      aux.add ( not (*it));
    if (D == aux) return true;
    return false;
}

  bool
  operator == (const orclause & D) {
/*{\Mbinop Ritorna se le 2 clausole sono uguali.}*/  
    if (size() != D.size() ) return false;
    orclause::iterator it;
    for (it = begin(); it != end(); it ++)
      if (D.count (*it) == 0) return false;
    return true;  
  }

  friend
  ostream &
  operator << (ostream & os, const orclause & x) {
/*{\Mbinopfunc Stampa su |os| le variabili presenti nella xorclause, nel 
formato: |\{<var1>,<var2>, ... ,<varn>\}|.
Utilizza |operator << (ostream & os, const var & v)| per stampare ogni elemento.}*/
    if (x.empty()) return os << "++ Empty Clause ++ ";
    orclause::iterator it = x.begin();
    os << "{";
    os << (*it);
    it ++;
    while (it != x.end()) { os << ";" << (*it); it ++;}
    os << "}";
    return os;
  }
					    

/*{\Mimplementation
Le Orclauses sono implementate tramite set, instanziata al tipo |var|.
La definizione della funzione |add| si \`e resa necessaria per analogia con la 
classe |xorclause|. Le funzioni |add()|, |is_complement()| hanno complessit\'a 
$O(1)$; gli operatori |<|, |==|, |<<| hanno complesit\'a $O(n)$, dove |n| \`e la lunghezza della clausola.}*/
};

#endif
