/*****************************************************************************
+	TAME 1.0
+
+	orclauset.h
 *****************************************************************************/

#ifndef TAME_ASSET_H
#define TAME_ASSET_H

#include "./orclause.h"
#include <algo.h>
#include <iostream.h>

#define SETMIN(A) (*((A).begin()))

/*{\Manpage {orclauset} {} {Orclausets} }*/

class orclauset : public set <orclause> {
/*{\Mdefinition Una istanza |C| del tipo di dati |\Mname| rappresenta un insieme
di clausole Or della logica booleana.}*/

  bool empty_clause; // flag che segnala la presenza di una clausola vuota

/*** Service Function Methods ***/

  void
  union_inserter (const orclauset & D) {
    orclauset Aux;
    set_union (begin(), end(), D.begin(), D.end(),
               inserter (Aux, Aux.begin() ) );
    *this = Aux;
  }

  void
  union_subsumpt (const orclauset & D) {
    orclauset::const_iterator itD;
    orclauset::iterator itC;
    bool present;
    for (itD = D.begin(); itD != D.end();itD++) {
      present = false;
      for (itC = begin(); itC != end();) {
        if (includes ((*itC).begin(), (*itC).end(),
                      (*itD).begin(), (*itD).end())) {
          if ((*itD).size() == (*itC).size()) {
            present = true;
            itC ++;
          }
          else {
            erase (itC++);
	  }  
	}  
	else {
	  itC ++;
	}  
      } //end for all clause in C
      if (!present)
        add (*itD);
    } // end for all clause in D
  }

  void
  fast_union_subsumpt (const orclauset & D) {
    orclauset::const_iterator itD;
    orclauset::iterator itC;
    bool present;
    var cmin, dmin;
    for (itD = D.begin(); itD != D.end(); itD ++) {
      present = false;
      dmin = SETMIN (*itD);
      for (itC = begin(); itC != end();) {
        cmin = SETMIN (*itC);
        if (dmin < cmin) // if dmin < cmin, from this clause of C,
          itC = end(); // there is no one containing dmin (because the order)
        else
          if (includes ((*itC).begin(), (*itC).end(),
                        (*itD).begin(), (*itD).end())) {
            if ((*itD).size() == (*itC).size()) {
              present = true;
              itC ++;
            }
            else
              erase (itC++);
	  }
	  else
	    itC ++;
      } //end for all clause in C
      if (!present)
        add (*itD);
    } // end for all clause in D
  }

/*** End Service Function Methods ***/

public:

/*{\Mcreation C }*/

   orclauset () {empty_clause = false;}
/*{\Mcreate crea una istanza di |\Mvar| e la inizializza con insiemi vuoti.}*/

/*{\Moperations 1.8 5}*/

   void
   set_bottom () {empty_clause = true;}
/*{\Mop Setta il flag che indica la presenza di una contraddizione.}*/

   void
   unset_bottom () {empty_clause = false;}
/*{\Mop Resetta il flag che indica la presenza di una contraddizione.}*/   

   bool
   bottom (){return (empty_clause);}
/*{\Mop Restituisce true se in |\Mvar| \'e presente una clausola 
contraddittoria, false altrimenti.}*/


/*{\Mtext \headerline{Updating Member Function} }*/

  void
  add (const orclause & c) {insert (c);}
/*{\Mop inserisce la clausola |c| nell'insieme di clausola |C|.}*/  


  void
  operator += (const orclauset & D) {
/*{\Mbinop Assegna a |\Mvar| l'unione insiemistica $C \cup  D$, effettuando la
sussunzione delle clausole contenute in |\Mvar|, ovvero:\\
$\forall d \in D:$\\
if $\exists c \in C : d \subset c, C.del (c), C.add (d)$\\
if $\exists c \in C : d = c$ skip to the next |d|\\
if $\not \exists c \in C : d \subset c, C.add (d)$
}*/
  fast_union_subsumpt (D);
  }

  orclauset 
  operator & (const orclauset & D) {
/*{\Mbinop Restituisce l'insieme intersezione insiemistica $C \cap D$}*/
    orclauset Aux;
    set_intersection (begin(), end(), D.begin(), D.end(), 
		      inserter (Aux, Aux.begin() ) );
    return Aux;
  }
  
/*{\Mtext \headerline{Output Extern Function} }*/

  friend
  ostream &
  operator << (ostream & os, const orclauset & C) {
/*{\Mbinopfunc Stampa su |os| le clausole presenti nella xorclauset |\Mvar|, 
separate da un accapo. Utilizza |operator << (ostream & os, orclause x)| per stampare ogni 
elemento.}*/
    if (C.empty()) return os << "** Empty Clause Set **" << endl;
    for (orclauset::iterator it = C.begin();
         it != C.end();
	 it ++)
       os << (*it) << endl;
    return os;
  }
					  

/*{\Mimplementation
Le Orclausets sono implementate tramite set, instanziata al tipo |orclause|.
Le funzioni |add()|, |bottom()|, |set_bottom()| hanno complessit\'a $O(1)$; le altre funzioni sono $O(n)$.}*/
};

#endif
