/*****************************************************************************
+	TAME 1.0
+
+	restart.h
 *****************************************************************************/

#include "./formula.h"
#include "./lassign.h"

/*{\Manpage {Algorithmic Guidelines for Formulas} {F} {Structural Functions} 
}*/

bool
init (formula & F, char * infile);
/*{\Mfuncl Inizializza la formula |\Mvar| e l'insieme delle variabili |v| che vi sono racchiuse, analizzando il file di Input descritto dalla stringa |infile|.
}*/

bool
evaluate (formula& G, const formula& F, var v);
/*{\Mfuncl  Assegna a |G| il valore della formula |\Mvar|, imponendo |v = true|.
Dove |v| \'e inteso con segno. |v| \'e aggiunta tra le variabili assegnate in
$G.Ass$.
}*/

bool
calculate (formula & F);
/*{\Mfuncl Esegue su |\Mvar| tutti i calcoli semplificativi, che non implicano
assunzione arbitraria di valori per alcuna variabile.
Ritorna |true| se non state generate clausole vuote nella computazione; 
|false| altrimenti.
Setta il flag di cambiamento di |F| in caso di semplificazione.
}*/


bool
lookahead(formula & F, lassign & VB); 
/*{\Mfuncl Esegue il calcolo del lookahead a 2 livelli sulla formula |\Mvar|:
\begin {itemize}
\item {controllo che non ci siano variabili che rendono falsa dopo un solo
passo la formula}
\item {inserimento delle clausole comuni ai 2 casi di assegnazione della
variabile nel caso non si renda falsa in nessuno dei 2 casi opposti la formula}\item {controllo che non venga generato un bottom() per entrambi i valori
assunti}
\end {itemize}
Restituisce $false$ nel caso si generi bottom() per entrambe le assunzioni,
$true$ altrimenti.
Setta il flag di cambiamento di |F|.}*/


bool
compute (formula & F, lassign & VB);
/*{\Mfuncl Funzione ad alto livello che esegue su |\Mvar| le operazioni
che non necessitano di split():
\begin {itemize}
\item {calcolo sulla formula}
\item {lookahead a 2 livelli}
\end {itemize}.
Restituisce |true| se il calcolo ha avuto esito positivo, 
|false| se si sono generati degli errori.
}*/

bool
computeone (formula & F, lassign & VB);
/*{\Mop Come la funzione precedente, in pi\'u aggiunge le assegnazioni che 
provengono dalla normale computazione (e non solo dalla |lookahead()| in |VB|.
Va usata all'inizio, quando si semplifica la formula di riferimento.}*/

bool
restart (formula & F, const formula & G, lassign & VB); 
/*{\Mfuncl Calcolo con split() sulla formula |G|, tenendo come riferimento la formula |F|.
Finch\'e possibile esegue la riduzione della formula |G|, senza effettuare 
branch. Ove necessario esegue il branch attraverso la |split()|.
In caso di sopraggiunta contraddizione, riassegna |G| con la formula di 
riferimento |F| riassegnando le variabili di branch, avendone preventivamente efettuato un riordine.
}*/

