/*****************************************************************************
+	TAME 1.0
+
+	var.h
 *****************************************************************************/

#ifndef TAME_VAR_H
#define TAME_VAR_H

#include <iostream.h>
#define TRUE (var (true))
#define FALSE (var (false))

#ifdef TAME_ON_DES
typedef short bit_t;
typedef unsigned char cycle_t;
typedef char block_t;
#define MIN_BIT 1
#define MAX_BIT 581
#define MIN_CYCLE 0
#define MAX_CYCLE 16
#define MIN_BLOCK 1
#define MAX_BLOCK 127

#else  // #!defined TAME_ON_DES
typedef short bit_t;
typedef unsigned short cycle_t;
typedef short block_t;
#define MIN_BIT 1
#define MAX_BIT 32767
#define MIN_CYCLE 0
#define MAX_CYCLE 65535
#define MIN_BLOCK 1
#define MAX_BLOCK 32768

#endif //TAME_ON_DES

/*{\Manpage {var} {} {Variabili} {v}}*/

class var {

/*{\Mdefinition
Una istanza |\Mvar| del tipo di dati |\Mname| rappresenta una variabile booleana
dotata di segno. \`E stata ideata per rappresentare la codificazione "human 
readable" nella riduzione in formule dell'algoritmo del DES. 
Per questo sono stati definiti i seguenti campi:
\begin {itemize}
        \item {nome variabile}
	\item {posizione occupata dal bit nel vettore}
        \item {numero ciclo}
        \item {numero blocco}
\end {itemize}
 }*/

protected:

/***** Data Structure (see Implementation) *****/
  char vname; /* var name (a valid character) */
  cycle_t vcycle; /* var number of e i flags */
  bit_t vbit;
  block_t vblock;
/***** End Data Structure *****/


/* Service Utility Functions */
  inline void 
  _set_name (char name) {vname = name; };

  void 
  _set_sign (bool sign) {
  if ((sign == false) && (vbit > 0))
    vbit = (- vbit); // set minus sign
  if ((sign == true) && (vbit < 0))
    vbit = (-vbit); //set plus sign
  return;
  }


  void 
  _set_cycle (int cycle_number) {
#ifdef TAME_DEBUG  
  if (cycle_number > MAX_CYCLE || cycle_number < MIN_CYCLE)
    cout << "Invalid cycle_number:" << cycle_number << endl;
#endif // TAME_DEBUG    
  vcycle = (cycle_t) cycle_number;
  return;
  }
  
  void 
  _set_bit (int bit_number) {
#ifdef TAME_DEBUG  
  if (bit_number > MAX_BIT || bit_number < MIN_BIT)
    cout << "Invalid bit_number:" << bit_number << endl;
#endif // TAME_DEBUG    
  vbit = (bit_t) bit_number;
  return;
  }

  
  void 
  _set_block (int block_number) {
#ifdef TAME_DEBUG  
  if (block_number > MAX_BLOCK || block_number < MIN_BLOCK)
    cout << "Invalid block_number:" << block_number << endl;
#endif // TAME_DEBUG    
  vblock = (block_t) block_number;
  return;
  }

  
/* End Service Utility Functions */  

public:
/*{\Mcreation v }*/

 var() {
/*{\Mcreate crea una instanza |\Mvar| of type |\Mname| and la inizializza con il valore "T" (True).}*/
  _set_name ('T');
  _set_bit (1);
  _set_cycle (0);
  _set_block (1);
  return;
}

  var (bool costant) {
/*{\Mcreate crea una instanza |\Mvar| of type |\Mname| and la inizializza con il valore costante contenuto in |costant|.}*/  
  _set_name ('T');
  _set_bit (1);
  _set_cycle (0);
  _set_block (1);
  _set_sign (costant);
  return;
  }


  var(bool sign, char name, int bit_number, int cycle_number, int block_number){
/*{\Mcreate crea una instanza |\Mvar| of type |\Mname| and la inizializzza con i valori passati come parametri.\\
\precond 	\\
		sign $= 1, 0$;\\
        	name $=$ lettera dell'alfabeto;\\
	        bit\_number $= [MIN\_BIT - MAX\_BIT]$;\\
	        cycle\_number $= [MIN\_CYCLE - MAX\_CYCLE]$;\\
	        block\_number $= [MIN\_BLOCK - MAX\_BLOCK]$;
}*/
  _set_name (name);
  _set_bit (bit_number);
  _set_cycle (cycle_number);
  _set_block (block_number);
  _set_sign (sign);
  return;
}


  var (const var & w) {
/*{\Mcreate crea una istanza |\Mvar| del tipo |\Mname| e la inizializza con il 
valore di $w$ (copy constructor).}*/  
  vname = w.name();
  vbit = w.vbit;
  vcycle = w.vcycle;
  vblock = w.block ();
}

/*{\Moperations 1.8 5}*/

  bool
  operator == (const var & w) const{
/*{\Mbinop ritorna |true| se |\Mname| $=$ |w|, |false| altrimenti.}*/
    if ( (vname == w.vname) &&
         (vbit == w.vbit) &&
	 (vcycle == w.vcycle) &&
	 (block() == w.block() ) ) return true;
    return false;
  }

  var &
  operator = (const var & w) {
/*{\Mbinop assegna a |\Mvar|  il valore di $w$}*/
  vname = w.vname;
  vcycle = w.vcycle;
  vbit = w.vbit;
  vblock = w.block();
  return *this;
}

  friend
  var 
  not (const var& w) {
/*{\Mfuncl    restituisce $w$ cambiato di segno. }*/
    var v = w;
    v.vbit = (-v.vbit);
    return v;
  }

  friend
  var
  abs (const var & w) {
/*{\Mfuncl restituisce il valore assoluto di |w|.}*/
    if (w.sign () ) return w;
    var v = w;
    v.vbit = (- w.vbit);
    return v;
  }  
  
  friend
  bool
  operator < (const var & a, const var & b) {
/*{\Mbinopfunc restituisce $0$ se |a| \`e maggiore od uguale a |b| effettuando la comparazione campo per campo nell'ordine:
\begin {itemize}
\item sign
\item block
\item cycle
\item bit
\item name
\end {itemize}
Altrimenti restituisce $1$.}*/
    if (a.block() < b.block() ) return true; 
    if (a.block() > b.block() ) return false;
    if (a.vcycle < b.vcycle) return true; 
    if (a.vcycle > b.vcycle) return false;
    if (a.bit() < b.bit() ) return true; 
    if (a.bit() > b.bit() ) return false;
    if (a.vname < b.vname) return true;
    if (a.vname > b.vname) return false;
    if (a.vbit < b.vbit) return true;
    if (a.vbit > b.vbit) return false;
    return false;
  }  
  
  bool
  sign() const {return (bool) (vbit > 0);}
/*{\Mop restituisce il segno di |\Mvar| nel senso della variabile.}*/


  char
  name() const {return vname;}
/*{\Mop restituisce il nome di |\Mvar| nel senso della variabile.}*/

  int
  bit() const {if (vbit < 0) return (int) (-vbit); return (int) vbit;}
/*{\Mop restituisce il numero di bit di |\Mvar| nel senso della variabile.}*/


  int
  cycle() const {return (int)(vcycle);}
/*{\Mop restituisce il numero di ciclo di |\Mvar| nel senso della variabile.}*/

  int
  block() const {if (vblock < 0) return (int) (-vblock); return (int) vblock;}
/*{\Mop restituisce il numero di blocco di |\Mvar| nel senso della variabile.}*/

  bool
  is_costant () const {return (bool) (name() == 'T');}
/*{\Mop restituisce se la variabile rappresenta una costante (|true| o |false|).}*/  

  bool
  is_negative () const {return (bool) (vbit < 0);}
/*{\Mop   restituisce |true| se la variabile ha segno negativo, |false| altrimenti.}*/

/*****************************
 Funzioni Esterne alla Classe
  *****************************/

  /*{\Mtext {\bf Output Functions}}*/

  friend
  ostream & 
  operator << (ostream & os, const var & v) {
/*{\Mbinopfunc stampa su |os| i valori della variabile. Utilizza un formato 
analogo a quello dell'input:\\
|[not](<name><bit>,<cycle>,<block>)|}*/
    if (v.is_costant()) {
      if (v.is_negative())
        os << "FALSE";
      else
        os << "TRUE";
      return os;
    }
    if ( v.is_negative()) {
      os << "NOT("
         << v.name()
         << v.bit() << ","
         << v.cycle() << ","
         << v.block() << ")";
    }
    else { // !v.is_negative()
      os << v.name()
         << v.bit() << ","
         << v.cycle() << ","
         << v.block();
    }
    return os;
  }


};
/*{\Mimplementation
Le variabili vengono rappresentate da una struct contenente:
\begin {enumerate}
\item   |char| nome della variabile
\item   |bit_t| numero di posizione della variabile nell'array
        assume i valori da $MIN\_BIT$ a $MAX\_BIT$
\item   |cycle_t| numero di ciclo
        assume i valori da $MIN\_CYCLE$ a $MAX\_CYCLE$
\item   |block_t| numero di blocco
        assume i valori da $MIN\_BLOCK$ a $MAX\_BLOCK$
\end {enumerate}
Per ognuno di detti tipi (con l'esclusione del tipo rappresentante il nome)
In questo modo diventa possibile usare codifiche differenti o anche aggiungerne di nuove.
}*/

/*{\Mtext
\bigskip
{\bf Complessit\'a computazionale delle funzioni}
\medskip

Ogni funzione ha complessit\'a $O(1)$.
}*/

/*{\Mtext
\bigskip
{\bf Indicazioni per la modifica degli intervalli di valori}
\medskip

Poich\`e |bit_t| e |block_t| assumono in generale solo valori positivi, la loro
negazione \`e stata utilizzata per rappresentare:
\begin {itemize}
\item |bit_t| $\rightarrow$ segno negativo della variabile
\item |block_t| $\rightarrow$ flag in uso nella classe derivata (|lvar|)
\end {itemize}
Onde non \`e possibile definire $MIN\_BIT$ o $MIN\_BLOCK$ $< 1$.
Non modificare le definizioni gi\'a esistenti, ma aggiungerne di nuove, 
modificando invece il Makefile generale.


\bigskip
{\bf Indicazioni per la definizione dei tipi}
\medskip

Questa classe \`e stata pensata in seno alla criptoanalisi, ed in particolare 
ad una sua applicazione sul DES. Per questo comprende un campo carattere
indicante il nome (la cui modifica richiede la revisione dell'intera classe).
I campi rappresentanti in indice hanno invece il tipo ridefinibile, modificando 
semplicemente la |typedef| corrispondente.
Occorre tuttavia osservare che la modifica deve essere fatta coerentemente con
i valori limite impostati (|MAX_...|).


\bigskip
{\bf Definizioni per il DES}
\medskip

I corretti valori sono caricati in fase di compilazione, definendo nel Makefile:
\newline
|TAME= TAME_ON_DES|
\newline
I corretti tipi sono definiti nell' header file:\\
|typedef short bit_t| \\
|typedef unsigned char cycle_t| \\
|typedef char block_t|.
}*/
#endif
