/*****************************************************************************
+	TAME 1.0
+
+	varset.h
 *****************************************************************************/

#ifndef TAME_VARSET_H
#define TAME_VARSET_H

#include <vector.h>
#include "./var.h"
#include <iostream.h>

/*{\Manpage {varset} {} {Set of Unassigned Boolean Variables} {V} }*/

class varset : public vector <var> {
/*{\Mdefinition
Una istanza |\Mvar| del tipo di dati |\Mname| rappresenta l'insieme delle
variabili, della logica booleana, presenti in qualche clausola ma non ancora 
assegnate. La sua definizione si \'e resa necessaria per semplificarne la ricerca in funzioni come |lookahead()| e |split()|.}*/


public:

/*{\Mcreation 3}*/

varset () {}
/*{\Mcreate crea una istanza di |\Mvar| e la inizializza all'insieme vuoto.
}*/

/*{\Moperations 1.8 5}*/

varset &
add (const var & a) {push_back(a); return *this;}
/*{\Mop inserisce la variabile |x| nell'insieme |\Mvar|.}*/

friend
ostream &
operator << (ostream & os, const varset & V) {
  for (varset::const_iterator itv = V.begin();
       itv != V.end();
       itv ++)
    os << (*itv) << endl;
    return os;
}

};

/*{\Mimplementation
I |\Mname| sono implementati come vector di |var|. Le operazioni prendono $O(1)$.
}*/
#endif //TAME_VARSET_H
