/*****************************************************************************
+	TAME 1.0
+
+	xorclause.h
 *****************************************************************************/

#ifndef TAME_XORCLAUSE_H
#define TAME_XORCLAUSE_H

#include "./var.h"
#include <set.h>

/*{\Manpage {xorclause} {} {Xorclauses} {X} }*/

class xorclause : public set <var> {
/*{\Mdefinition
Una istanza |X| del tipo di dati |\Mname| rappresenta una clausola Xor della logica booleana.
}*/


public:

/*{\Mcreation 3 }*/

   xorclause () {}
/*{\Mcreate crea una istanza di |\Mvar| e la inizializza con insiemi vuoti.
}*/
   

/*{\Moperations 1.8 5}*/

void add (const var& x) {
/*{\Mop inserisce la variabile |x| nella clausola |X|, tenendo conto se la 
variabile |x| sia gi\'a presente (asserita o negata) all'interno della 
|xorclause|.}*/
  if (x == FALSE) return;
  if (count (x) != 0 ) {
    erase (x); 
    return;}
  if (count (not(x)) != 0 )
    { erase (not(x));
      add (TRUE);
      return;
    }
  insert (x);
  return;
  }
 
  friend
  xorclause
  not (const xorclause & Y) {
/*{\Mfuncl Restituisce la xclause cambiata di segno, ovvero:\\
elimina la costante |True| se presente
altrimenti nega la prima variabile.}*/
    xorclause aux = Y;
    xorclause::iterator it;
    if ( (it = aux.find (TRUE)) != aux.end() ) aux.erase (it);
    else {
      it = aux.begin();
      var temp = (*it);
      aux.erase (it);
      aux.insert (not (temp));
    }  
    return aux;
  }

  bool
  operator == (const xorclause & D) {
/*{\Mbinopl Ritorna se le 2 clausole sono uguali.}*/
    if (size() != D.size() ) return false;
    xorclause::iterator it;
    for (it = begin(); it != end(); it ++)
      if (D.count (*it) == 0) return false;
    return true;
  }
			      
  friend
  ostream &
  operator << (ostream & os, const xorclause x) {
/*{\Mbinopfunc Stampa su |os| le variabili presenti nella xorclause, nel 
formato:
|\{<var1>,<var2>, ... , <varn>\}|.
Utilizza |operator << (ostream & os, const var & v)| per stampare ogni elemento.}*/
    if (x.empty()) return os << "++ Empty Xclause ++";
    xorclause::iterator it = x.begin();
    os << "{";
    os << (*it);
    it ++;
    while (it != x.end()) { os << ";" << (*it); it ++;}
    os << "}";
    return os;
  }  
  
/*{\Mimplementation
Le Xorclauses sono implementate tramite set, instanziata al tipo |var|.
La definizione della funzione |add| si \`e resa necessaria poich\`e si \`e dovuta ridefinire la funzione di inserimento, in relazione alla presenza o meno della variabile da inserire all'interno della |xorclause|.
Infatti, per non trattare le |xorclause| come multiinsiemi, nell'inserimento si deve eseguire uno |xor| e non un |or|, ovvero si possono presentare i 3 casi, per l'inserimento della variabile |x| nella xorclause |X|:\\
$x \not\subset X$ e $not(x) \not\subset X$, onde $x \oplus X \equiv x \lor X$\\
$x \subset X$ e $not(x) \not\subset X$, onde $x \oplus X \equiv X - x$\\
$not (x) \subset X$ e $x \not\subset X$, onde $x \oplus X \equiv True \lor X$\\

\bigskip
{\bf Complessit\'a delle funzioni}
\medskip

La funzione add ha complessit\'a $O(\log n)$, la funzione |not()| e gli operatori |<|, |==|, |<<| hannocomplessit\'a $O(n)$, dove |n| \`e il numero dei letterali di |\Mvar|.
}*/


};
#endif
