/*****************************************************************************
+	TAME 1.0
+
+	xorclauset.h
 *****************************************************************************/

#ifndef TAME_XORCLAUSET_H
#define TAME_XORCLAUSET_H

#include "./xorclause.h"
#include <algo.h>

/*{\Manpage {xorclauset} {} {Xorclausets} }*/

class xorclauset : public set <xorclause> {
/*{\Mdefinition
Una istanza |X| del tipo di dati |\Mname| rappresenta un insieme di clausole Xor
della logica booleana.}*/

  bool empty_clause; // flag che segnala la presenza di una clausola vuota

public:

/*{\Mcreation C }*/

   xorclauset () {empty_clause = false;}
/*{\Mcreate crea una istanza di |\Mvar| e la inizializza con insiemi vuoti.}*/

/*{\Moperations 1.8 5}*/

   void
   set_bottom () {empty_clause = true;}
/*{\Mop Setta il flag che indica la presenza di una contraddizione.}*/

   void
   unset_bottom () {empty_clause = false;}
/*{\Mop Resetta il flag che indica la presenza di una contraddizione.}*/   

   bool
   bottom () {return (empty_clause);}
/*{\Mop Restituisce true se in |\Mvar| \'e presente una clausola contraddittoria, false altrimenti.}*/

/*{\Mtext \headerline{Updating Member Function} }*/

  void
  add (const xorclause & x) {insert (x);}
/*{\Mop inserisce la clausola |x| nell'insieme di clausola |C|.}*/


  void
  operator += (const xorclauset & D) {
/*{\Mbinop Assegna a |\Mvar| l'unione insiemistica $C \cup D$}*/
    set_union (begin(), end(), D.begin(), D.end(),
               inserter (*this, this->begin() ) );
  }

  xorclauset
  operator & (const xorclauset & D) {
/*{\Mbinop Restituisce l'insieme intersezione insiemistica $C \cap D$}*/
    xorclauset Aux;
    set_intersection (begin(), end(), D.begin(), D.end(),
                      inserter (Aux, Aux.begin() ) );
    return Aux;
  }


/*{\Mtext \headerline{Output Extern Function} }*/

  friend
  ostream &
  operator << (ostream & os, const xorclauset & X) {
/*{\Mbinopfunc Stampa su |os| le clausole presenti nella xorclauset |\Mvar|,
separate da un accapo. Utilizza |operator << (ostream & os, xorclause x)| per 
stampare ogni elemento.}*/
    if (X.empty()) return os << "** Empty Xclause Set **" << endl;
    for (xorclauset::iterator it = X.begin();
         it != X.end();
         it ++)
       os << (*it) << endl;
    return os;
  }
					     

/*{\Mimplementation
Le Xorclauses sono implementate tramite set, instanziata al tipo |xorclause|.
Le funzioni |add()|, |bottom()|, |set_bottom()| hanno complessit\'a $O(1)$; le altre funzioni sono $O(n)$.}*/
};

#endif // TAME_XORCLAUSET_H
