/*****************************************************************************
+	TAME 1.0
+
+	xorque.h
 *****************************************************************************/

#ifndef TAME_XORQUE_H
#define TAME_XORQUE_H

#include <deque.h>
#include "./xorclause.h"
#include "./lassign.h"

/*{\Manpage {xorque} {} {Xorqueues} {XQ}}*/

class xorque: public deque <xorclause> {

  bool empty_clause;
  
  void
  reduce (xorclause & x) {
    if (x.size () < 2)
      return;
    xorclause::iterator itx = x.find (TRUE);
    if (itx != x.end()) {
      x.erase (itx);
      x = not (x);
    }  
  }

  
  public:

/*{\Mcreation 3}*/

  xorque() {empty_clause = false;}
/*{\Mcreate crea una istanza di |\Mvar| e la inizializza all'insieme vuoto.
}*/

/*{\Moperations 1.8 5}*/

  bool
  bottom () {return empty_clause;}
/*{\Mop restituisce se \'e sorta una contraddizione.}*/

  void
  set_bottom () {empty_clause = true;}
/*{\Mop Setta la condizione di errore.}*/

  void
  unset_bottom () {empty_clause = false;}
/*{\Mop resetta la condizione di errore.}*/  

  void
  Add (const var & a, const var & b) {
/*{\Mop inserisce la clausola |x| nello stack come ultimo elemento.}*/
    xorclause temp;
    temp.add (a);
    temp.add (not(b));
    reduce (temp);
    push_back (temp);
  }  

  void
  Push (const xorclause & x) {push_back (x);}
/*{\Mop inserisce la clausola |x| nello stack come ultimo elemento.}*/

  const xorclause &
  Top () const {return front ();}
/*{\Mop ritorna il riferimento al primo elemento.}*/

  void
  Pop () {pop_front();}
/*{\Mop cancella il primo elemento.}*/  

  void
  AddAllArbitraryAssignment (const lassign & VB) {
/*{\Mop Inserisce tutte le assegnazioni di |VB| in |\Mvar|.}*/
    lassign::const_iterator itvb;
    for (itvb = VB.begin(); itvb != VB.end(); itvb ++) {
      this-> Add ((*itvb).lhs, (*itvb).rhs);
    }
  }  

  void
  AddAllAssignment (const assignmap & Ass) {
/*{\Mop Inserisce tutte le assegnazioni di |Ass| in |\Mvar|.}*/
    assignmap::const_iterator itass;
    for (itass = Ass.begin(); itass != Ass.end(); itass ++) {
      this->Add ((*itass).first, (*itass).second);
    }  
  }
	

/*{\Mtext \headerline{Output Friend Function} }*/  
  
  friend
  ostream &
  operator << (ostream & os, const xorque & XQ) {
/*{\Mbinopfunc Stampa su |os| le clausole presenti nella XorQue |\Mvar|,
separate da un accapo. Utilizza |operator << (ostream & os, xorclause x)| per
stampare ogni elemento.}*/
    if (XQ.empty() ) return (os << "** Empty XorQue **\n");
    xorque::const_iterator itxq;
    for (itxq = XQ.begin(); itxq != XQ.end(); itxq ++) {
      os << (*itxq) << endl;
    }
    return os;
  }

/*{\Mimplementation
Le XorQue sono implementate tramite deque, instanziata al tipo |xorclause|.
Tutte le funzioni hanno complessit\'a $O(1)$, con l'eccezione di |<<| che \'e
$O(n)$.}*/

};

#endif // TAME_XORQUE_H
