/*******************************************************************************
+	TAME 1.0
+
+	formula.C
*******************************************************************************/


#include "../include/formula.h"
#include "../include/rules.h"

formula&
formula::operator= (const formula& G) {
 C = G.C;
 C.unset_bottom ();
 X = G.X;
 X.unset_bottom ();
 Ass = G.Ass;
 S = G.S;
 S.unset_bottom ();
 V.copy (G.V);
 return (*this);
}  

bool
formula::operator!= (const formula & G) {
  if ( C != G.C ||
       X != G.X ||
       S != G.S ||
       Ass != G.Ass)
   return true;
  return false; 
}

formula &
formula::operator += (const formula & G) {
  C += G.C;
  X += G.X;
  S += G.S;
  resync_vset();
  unset_changed();
  return (*this);
}

formula 
formula::operator & (const formula & G) {
  formula Aux;
  Aux.C = C & G.C;
  Aux.X = X & G.X;
  Aux.S = S & G.S;
  Aux.Ass = Ass & G.Ass;
  Aux.resync_vset();
  return Aux;
}

ostream &
operator << (ostream & os, const formula & F) {
  os << "## OR Clauses Set" << endl;
  os << F.C;
  os << "## XOR Clauses NOT Selected Set" << endl;
  os << F.X;
  os << "## Selected XOR CLauses Set" << endl;
  os << F.S;
  os << "## Assignment Set" << endl;
  os << F.Ass;
  os << "## Occurrences Set" << endl;
  os << F.V;
  return os;
} 

void
formula::populate_fromOR (vset & V, const orclauset & C) {
  orclauset::iterator itc;
  orclause::iterator it;
  for (itc = C.begin(); itc != C.end(); itc ++) {
    for (it = (*itc).begin(); it != (*itc).end(); it ++) {
      V.addor ( (*it) ); // increment occourrencies of atom in V
    } // end for all proposition in the current clause
  } // end for all the clauses
}

void
formula::populate_fromXOR (vset & V, const xorclauset & X) {
  xorclauset::const_iterator itx;
  xorclause::const_iterator it;
  for (itx = X.begin(); itx != X.end(); itx ++) {
    for (it = (*itx).begin(); it != (*itx).end(); it ++) {
      V.addxor ( abs(*it) ); // increment occourrencies of atom in V
    } // end for all proposition in the current clause
  } // end for all the clauses
}

void
formula::populate_fromSEL (vset & V, const selectxor & S) {
  selectxor::const_iterator its;
  xorclause::const_iterator it;
  for (its = S.begin(); its != S.end(); its ++) {
    V.addsel (abs ((*its).first));
    for (it = ((*its).second).begin(); it != ((*its).second).end(); it ++)
      V.addsel (abs (*it));
  }
}

void
formula::populate_fromASS (vset & V, const assignmap & Ass) {
  assignmap::const_iterator itass;
  for (itass = Ass.begin(); itass != Ass.end(); itass ++) {
    V.addass ((*itass).first);
    V.addass ((*itass).second);
  }
}  

bool
formula::resync_vset () {
  V.flush();
  populate_fromOR (V, C);
  populate_fromXOR (V, X);
  populate_fromSEL (V, S);
  populate_fromASS (V, Ass);
  return true;
}  

bool
formula::occur (var a) const {
  vset::const_iterator itv;
  itv = V.find (abs (a));
  if (itv != V.end())
    if ( (*itv).second.get_split_occurrences() > 0)
      return true;
  return false;
}  

void
formula::AddOr (const orclause & c) {
  orclause::const_iterator it;
  for (it = c.begin(); it != c.end(); it++)
    V.addor (*it);
  C.add (c);
}

void 
formula::AddXor (const xorclause & x) {
  xorclause::const_iterator it;
  for (it = x.begin(); it != x.end(); it++)
    V.addxor (*it);
  X.add (x);
}

void
formula::AddAssign (var A, var B) {
  if ( (S.find (abs (A) ) != S.end() )) Unselect ((*this), A);
  if ( (S.find (abs (B) ) != S.end() )) Unselect ((*this), B);
  Ass.add (A, B);
  V.addass (A);
  V.addass (B);
}
	  
void
formula::AddSel (const var v, const xorclause & x) {
  xorclause::iterator it;
  for (it = x.begin(); it != x.end(); it++)
    V.addsel (*it);
  S.add (v, x);
}

void
formula::DelOr (const orclause & c) {
  orclause::const_iterator it;
  for (it = c.begin(); it != c.end(); it++)
    V.subor (*it);
  C.erase (c);
}

void
formula::DelOr (orclauset::iterator itc) {
  orclause::const_iterator it;
  for (it = (*itc).begin(); it != (*itc).end(); it++)
    V.subor (*it);
  C.erase (itc);
}

void
formula::DelXor (const xorclause & x) {
  xorclause::const_iterator it;
  for (it = x.begin(); it != x.end(); it++)
    V.subxor (*it);
  X.erase (x);
}

void
formula::DelXor (xorclauset::iterator itx) {
  xorclause::const_iterator it;
  for (it = (*itx).begin(); it != (*itx).end(); it++)
    V.subxor (*it);
  X.erase (itx);
}

void
formula::DelSel (selectxor::iterator its) {
  xorclause::iterator it;
  xorclause aux = (*its).second;
  for (it = aux.begin(); it != aux.end(); it++)
    V.subsel (*it);
  V.subsel ( abs ((*its).first)); // the abs() isn't necessary! 
  S.erase (its);
}

void
formula::PrintSolution (ofstream & of) {
  if (of.is_open() )
    printsolution (of);
  else
    printsolution (cout);
}

void
formula::printsolution (ostream & os) {
  solution_number ++;
  os << "After " << get_restart() << " cycles of restart,";
  os << "the " << get_solution_number() << " solution:" << endl;
  os << "Explicit Assignments\n" << Ass << endl;
  os << "Xor Clauses\n" << X << endl;
  os << "Definitions in Selected Xor Clauses\n" << S << endl;
}

void
formula::PrintInsolution (ofstream & of) {
  if (of.is_open() )
    printinsolution (of);
  else
    printinsolution (cout);
}

void
formula::printinsolution (ostream & os) {
  os << "Insatisfiable Formula!\n";
}  
