/******************************************************************************
+	TAME 1.0
+
+	lookahead.C
 *****************************************************************************/

#include <iostream.h>
#include "../include/varset.h"
#include "../include/restart.h"
#include "../include/logger.h"

extern
bool LOOKAHEAD_2LEVEL;

extern
tlogger logger;

extern
bool
Simp (formula & F, xorque & XQ);

bool
evaluate (formula & G, const formula & F, var v) { 
  xorque XQ; // for storing the assign a := TRUE
  G = F;
  XQ.Add (v, TRUE);
  logger (6, "Evaluate: Simplifiing tide");
  Simp (G, XQ);
  if (!G.bottom()) {
    logger (6, "Evaluate: Calculating tide");
    calculate (G);
  }  
  return (!G.bottom());
}

bool
lookahead(formula & F, lassign & VB) { 
  formula Fneg, Fpos; // 2 comodity formulas
  formula Ftemp; // formula for intersection (Fpos & Fneg)
  formula Fint; // formula for difference (Ftemp - F)
  xorque XQ; // for store the new common assignment
  var v; // current working var
  F.set_lookahead(0);
  int cicle = 0;
  logger (1, "LookAhead Function");
      
  do {
    F.unset_changed();
    vset::iterator itv;
    cicle ++;
    logger (2, "Beginning Lookahead cycle", cicle);
    logger (6, "Size of OR Clauses ", F.C.size());
    logger (6, "Size of XOR Clauses ", F.X.size());
    logger (6, "Size of Selected  XOR", F.S.size());
    logger (6, "Size of Assignment", F.Ass.size());
    logger (6, "Number of Variables", F.V.size());
    for (itv = F.V.begin();
         itv != F.V.end(); itv ++ ) {
      if (F.occur((*itv).first)) { // if v occur in F
	v = (*itv).first;
	logger (3, "Looking Ahead the var", v);
	logger (6, "with occurrences", (*itv).second.get_abs_occurrences());
	logger (3, "Evaluating Fneg, asserting the var", not(v));
        evaluate (Fneg, F, not(v)); // evaluate F with premise: "v := false"
	logger (9, "Fneg:", Fneg);
	logger (3, "Evaluating Fpos, asserting the var", v);
        evaluate (Fpos, F, v); // evaluate F with premise: "v := true"
	logger (9, "Fpos:", Fpos);
        if ( Fneg.bottom() && Fpos.bottom () ) {
          logger (2, "The 2 evaluation give bottom on", abs((*itv).first));
	  return false;
        }  // endif ( Fneg.bottom() && Fpos.bottom () )
        if ( Fneg.bottom() && (!Fpos.bottom()) ) {
          logger (2, "Fneg -> bottom, Fpos ok, on", abs ((*itv).first));
	  F = Fpos;
	  logger (9, "Formula F = Fpos", F);
	  F.set_changed();
          F.V.del (v);
          VB.add (v, TRUE, false); // insert v in the assign variables 
	  logger (5, "The BackTrack Variables Set", VB);
        } // endif ( Fneg.bottom() && (!Fpos.bottom ()) )
        if ((!Fneg.bottom ())&& Fpos.bottom () ) {
          logger (2, "Fpos -> bottom, Fneg ok on", abs((*itv).first));
	  F = Fneg;
	  logger (9, "Formula F = Fneg:" , F);
	  F.set_changed();
          F.V.del (v);
          VB.add (v, FALSE, false); // insert not(v) in the assign variables 
	  logger (5, "The BackTrack Variables Set", VB);
        } // endif ((!Fneg.bottom ())&& Fpos.bottom () )

  if (LOOKAHEAD_2LEVEL) {
        if ( (!Fneg.bottom ()) && (!Fpos.bottom ()) ) {
          logger (2, "Fpos, Fneg ok on", abs((*itv).first));
	  Ftemp = ( Fneg & Fpos );
	  logger (9, "Common Entities (Fpos & Fneg)", Ftemp);
	  if (!Ftemp.empty()) {
	      Fint.Ass = (Ftemp.Ass - F.Ass);
	      logger (9, "Difference Set (Ftemp - F)", Fint);
	      XQ.AddAllAssignment (Fint.Ass);
	      logger (5, "New Assignment:", XQ);
	      Simp (F, XQ);
	      logger (9, "Formula Simplified");
	      F += Ftemp;
	      logger (3, "Size of F.C ", F.C.size());
	      logger (3, "Size of F.X ", F.X.size());
	      logger (3, "Size of F.S ", F.S.size());
	      logger (3, "Size of F.Ass ", F.Ass.size());
	      // F.set_changed();
	      logger (9, "Formula F", F);
	  } // endif (!Ftemp.empty())  
	  else
	    logger (5, "(Fpos & Fneg) == {}");
        }  // endif (!Fneg.bottom ()) && (!Fpos.bottom ()) )
  } // endif LOOKAHEAD_2LEVEL

      }  // end if (!F.occur (v))
      else {
        logger (2, "The var not occur!", (*itv).first);
      }	
    }
    logger (2, "Exiting LookAhead cycle", cicle);
  } while (F.changed() );

  F.set_lookahead(cicle);
  if (cicle > 1)
    F.set_changed();
  logger (1, "Exiting LookAhead Function");  
  return true;
}

