/*****************************************************************************
+	TAME 1.0
+
+	redXor.C
 *****************************************************************************/

#include "../../include/rules.h"

bool
odd (const int n) {
// return true if n is odd, false otherwise
  if ( ((n/2) * 2) == n) return false;
  return true;
}
    

inline
static
bool
firstRed (xorclause & c) {
  /* first. eliminate TRUE costant: T + L + C -> -L + C*/
  xorclause::iterator it;
  int elimination = 0;
  while ( ((it = c.find(TRUE) ) != c.end() ) && (c.size() > 1) ) {
    c.erase (it); // erase TRUE constant
    elimination ++;
  }  
  if (odd(elimination)) {
    it = c.begin();
    c = not (c); //negate the first var
  }
  return (elimination != 0);
}

inline
static
bool
secondRed (xorclause & c) {
  /* second. eliminate FALSE costant: F + C -> C */
  xorclause::iterator it;
  int elimination = 0;
  while ((it = c.find (FALSE)) != c.end() ) {
    c.erase (it);
    elimination ++;
  }
  return (elimination != 0);
}

  /* third. eliminate same-sign redundancies: L + L + C -> C */
  /* it is void because the xorclause are set, so it not possible that it is
     more than 1 occurrence of L in the same orclause
   */

inline
static
bool
fourthRed (xorclause & c) {
  /* fourth. eliminate toggle-sign redundancies: L + -L + C -> T + C */
  xorclause::iterator it, itr;
  var A;
  for (it = c.begin ();
       it != c.end();) {
    A = (*it);
    itr = c.find (not (A) );
    if (itr != c.end () ) {
      c.erase (itr);  // delete -L
      c.erase (it ++); // delete L
      c.add (TRUE);
      return true;
    }
    else it ++;
  }
  return false;
}

#ifdef TAME_I_WOULD_FIVETH_REDUCTION
inline
static
bool
fivethRed (xorclause & c) {
  /* fiveth. eliminate sign redundancies: -A + -B + C -> A + B + C */
  xorclause::iterator it;
  xorclause temp;
  int negative = 0;
  for (it = c.begin(); it != c.end(); it ++) {
    temp.add (abs(*it));
    if ((*it).is_negative())
      negative ++;
  }
  if (odd (negative))
    c = not (temp);
  else 
    c = temp;
  if (negative > 1)
    return true;
  else return false;  
}

#endif 

bool
RedXor (xorclause & x) {
  int rednumber;
  rednumber = 0; //reset the reduce-iteration conter
  do {
    rednumber ++; // set counter
  } while ( firstRed (x) ||
            secondRed (x) ||
            fourthRed (x) ); // until reduce the clause
  if (rednumber > 1) //it was a profit reduction
    return true;
  return false;
}
