/***********************************************************************
+	TAME 1.0
+
+	main.C Principle Procedure
+	Call the Functions:
+	  * init() defined in ./init.C (inizialite reference formula)
+	  * computeone() to simplifiing the beginning reference formula
+	  * restart() defined in ./restart.C (begin the calculation)
+	Declare the global variable:
+	  * formula reference, tide
+	  * lassign VB
+	Define the Options from prompt
+	  * getopt 
***********************************************************************/

#include <iostream.h>
#include <fstream.h>
#include <getopt.h>
#include <string.h>
#include "../include/formula.h"
#include "../include/lassign.h"
#include "../include/restart.h"
#include "../include/tame.h"
#include "../include/logger.h"
#include "../include/storer.h"
#include "../include/heuristics.h"

/**************** GLOBAL PROGRAM VARIABLES *****************************/
char infile [120] = ""; // the name of the file containing the input formula
ofstream outputfile; // the file to write solution, etc.
ofstream logfile;
tlogger logger;
tstorer storer;
int verbose = 0; // store the verbosity level; default 0 (minimum)

bool ALL_SOLUTIONS = false; // store if you want the first solution or all
bool COMPUTE_WITH_LOOKAHEAD = true; // store if you want lookahead during normal
                       // computation (in compute function)
bool INIT_WITH_LOOKAHEAD = true; // store if you want initial simplification
		      // with lookahead
bool LOOKAHEAD_2LEVEL = false; // store if you want 2 level lookahead 
bool NO_GAUSS = false; // store if you dont want Gauss Resolution
bool SELECTIVE_SPLIT = false; // store if you want split only on one kind of var

char splitting_char = 'K';
tpriority priority; // priority function for splitting

int 
main(int argc, char * argv[]) {
/***** Principle Variables Declaration **********/
  formula reference; // reference formula
  formula tide; // working formula
  lassign VB; // labeled backtrack assigned variables

/***** Command Line Option Treatment *****/
  int optch;
  static char optstring [] = "I:O:S:Vabce:ghls:x:";

  while ( (optch = getopt (argc, argv, optstring)) != -1)
    switch (optch) {
      case 'I':
        if (strlen (optarg) != 0) strcpy (infile, optarg);
        break;
      case 'O':
        if (strlen (optarg) != 0) 
	  outputfile.open (optarg, ios::app);
	if (!outputfile.is_open()) {
	  cerr << "Cannot open output file" << endl;
	  exit (0);
	}  
        break;
      case 'S':
        if (strlen (optarg) != 0) {
          storer.set_storage_file (optarg);
        }
        break;
      case 'V':
        print_version();
	break;
      case 'a':
        ALL_SOLUTIONS = true;
        break;
      case 'b':
        COMPUTE_WITH_LOOKAHEAD = false;
	break;
      case 'c':
        INIT_WITH_LOOKAHEAD = false;
	break;
      case 'e':
        if (strlen (optarg) != 0)
	  priority.set_heuristic(optarg);
	else { 
	  print_usage();  
	  exit (0);
	}  
	break;  
      case 'g':
        NO_GAUSS = true;
	break;
      case 'h':
        print_usage();
        exit (0);
      case 'l':
        LOOKAHEAD_2LEVEL = true;
	break;
      case 's':
        if (strlen (optarg) != 0) {
          SELECTIVE_SPLIT = true;
	  splitting_char = optarg [0];
	}
	break;
      case 'x':
        if (strlen (optarg) != 0) {
	  verbose = atoi (optarg);
	  logger.set_debug_level (verbose);
	}  
	break;
      default:
        print_usage();
        exit (0);
    }

  logfile.open ("./tame.log"); // open standard log file
  if (!logfile.is_open())
    cerr << "Can't open logfile" << endl;
  logger.set_auxiliary (&logfile);
  logger.enable_auxiliary();

  PrintStartTime (outputfile);

/*** Initialization ****/
  if (init (reference, infile) == 0) {
    logger (0, "Cannot Initializate the Reference Formula");
    exit (0);
  }  
  logger (9, "Tame: Formula reference", reference);


/*** Calculation ***/

  if (computeone (reference, VB) ) {
    tide = reference;
    restart(tide, reference, VB);
  }
  else // not have solution
    tide.PrintInsolution (outputfile);
  PrintStopTime (outputfile);  
  return 1; 
}
